@@ -28,7 +28,6 @@
 //
 
 using System;
-using System.Linq;
 using System.Collections.Generic;
 using Mono.Unix;
 using Gtk;
@@ -67,10 +66,8 @@ namespace Nereid
         private HPaned views_pane;
         private ViewContainer view_container;
         private VBox source_box;
-        private Widget track_info_container;
         private CoverArtDisplay cover_art_display;
         private Widget cover_art_container;
-        private ConnectedSeekSlider seek_slider;
 
         // Major Interaction Components
         private SourceView source_view;
@@ -94,18 +91,6 @@ namespace Nereid
         {
         }
 
-        private int? original_seek_width;
-        private void SetSimple (bool simple)
-        {
-            main_menu.Visible =
-            source_box.Visible =
-            footer_toolbar.Visible =
-            track_info_container.Visible = !simple;
-
-            original_seek_width = original_seek_width ?? seek_slider.SeekSlider.WidthRequest;
-            seek_slider.SeekSlider.WidthRequest = original_seek_width.Value + (simple ? 100 : 0);
-        }
-
         public PlayerInterface () : base (Catalog.GetString ("Banshee Media Player"), "player_window", 1024, 700)
         {
             // if (PlatformDetection.IsMeeGo) {
@@ -200,7 +185,7 @@ namespace Nereid
 
             if (PlatformDetection.IsMeeGo) {
                 header_toolbar.IconSize = IconSize.LargeToolbar;
-                header_toolbar.Name = "meego-toolbar";
+                header_toolbar.Name = "moblin-toolbar";
             }
 
             toolbar_alignment.Add (header_toolbar);
@@ -214,15 +199,15 @@ namespace Nereid
             next_button.Show ();
             ActionService.PopulateToolbarPlaceholder (header_toolbar, "/HeaderToolbar/NextArrowButton", next_button);
 
-            seek_slider = new ConnectedSeekSlider ();
+            ConnectedSeekSlider seek_slider = new ConnectedSeekSlider ();
             seek_slider.Show ();
             ActionService.PopulateToolbarPlaceholder (header_toolbar, "/HeaderToolbar/SeekSlider", seek_slider);
 
             var track_info_display = new ClassicTrackInfoDisplay ();
             track_info_display.Show ();
-            track_info_container = TrackInfoDisplay.GetEditable (track_info_display);
-            track_info_container.Show ();
-            ActionService.PopulateToolbarPlaceholder (header_toolbar, "/HeaderToolbar/TrackInfoDisplay", track_info_container, true);
+            var editable = TrackInfoDisplay.GetEditable (track_info_display);
+            editable.Show ();
+            ActionService.PopulateToolbarPlaceholder (header_toolbar, "/HeaderToolbar/TrackInfoDisplay", editable, true);
 
             if (PlatformDetection.IsMeeGo) {
                 track_info_display.ArtworkSpacing = 5;
@@ -268,7 +253,7 @@ namespace Nereid
                 };
                 source_scroll.Add (source_view);
 
-                var color = new Gdk.Color ((byte)0xe6, (byte)0xe6, (byte)0xe6);
+                var color = new Gdk.Color ((byte)0xdd, (byte)0xdd, (byte)0xdd);
                 Gdk.Colormap.System.AllocColor (ref color, true, true);
                 source_view.ModifyBase (StateType.Normal, color);
             } else {
@@ -395,10 +380,7 @@ namespace Nereid
 
             source_view.RowActivated += delegate {
                 Source source = ServiceManager.SourceManager.ActiveSource;
-                var handler = source.Properties.Get<System.Action> ("ActivationAction");
-                if (handler != null) {
-                    handler ();
-                } else if (source is ITrackModelSource) {
+                if (source is ITrackModelSource) {
                     ServiceManager.PlaybackController.NextSource = (ITrackModelSource)source;
                     // Allow changing the play source without stopping the current song by
                     // holding ctrl when activating a source. After the song is done, playback will
@@ -443,11 +425,6 @@ namespace Nereid
                 view_container.SearchEntry.Ready = false;
                 view_container.SearchEntry.CancelSearch ();
 
-                /* Translators: this is a verb (command), not a noun (things) */
-                var msg = source.Properties.Get<string> ("SearchEntryDescription") ?? Catalog.GetString ("Search");
-                view_container.SearchEntry.EmptyMessage = msg;
-                view_container.SearchEntry.TooltipText = msg;
-
                 if (source.FilterQuery != null) {
                     view_container.SearchEntry.Query = source.FilterQuery;
                     view_container.SearchEntry.ActivateFilter ((int)source.FilterType);
@@ -478,8 +455,6 @@ namespace Nereid
 
                 UpdateSourceInformation ();
                 view_container.SearchEntry.Ready = true;
-
-                SetSimple (source.Properties.Get<bool> ("Nereid.SimpleUI"));
             });
         }
 
@@ -494,17 +469,11 @@ namespace Nereid
 
                 case "FilterQuery":
                     var source = ServiceManager.SourceManager.ActiveSource;
-                    var search_entry = source.Properties.Get<SearchEntry> ("Nereid.SearchEntry") ?? view_container.SearchEntry;
-                    if (!search_entry.HasFocus) {
-                        ThreadAssist.ProxyToMain (delegate {
-                            view_container.SearchEntry.Ready = false;
-                            view_container.SearchEntry.Query = source.FilterQuery;
-                            view_container.SearchEntry.Ready = true;
-                        });
-                    }
-                    break;
-                case "Nereid.SimpleUI":
-                    SetSimple (ServiceManager.SourceManager.ActiveSource.Properties.Get<bool> ("Nereid.SimpleUI"));
+                    ThreadAssist.ProxyToMain (delegate {
+                        view_container.SearchEntry.Ready = false;
+                        view_container.SearchEntry.Query = source.FilterQuery;
+                        view_container.SearchEntry.Ready = true;
+                    });
                     break;
             }
         }
@@ -553,8 +522,6 @@ namespace Nereid
             view_container.Header.Visible = source.Properties.Contains ("Nereid.SourceContents.HeaderVisible") ?
                 source.Properties.Get<bool> ("Nereid.SourceContents.HeaderVisible") : true;
 
-            view_container.SetTitleWidget (source.Properties.Get<Widget> ("Nereid.SourceContents.TitleWidget"));
-
             Widget header_widget = null;
             if (source.Properties.Contains ("Nereid.SourceContents.HeaderWidget")) {
                 header_widget = source.Properties.Get<Widget> ("Nereid.SourceContents.HeaderWidget");
@@ -693,11 +660,8 @@ namespace Nereid
                     break;
             }
 
-            // The source might have its own custom search entry - use it if so
-            var src = ServiceManager.SourceManager.ActiveSource;
-            var search_entry = src.Properties.Get<SearchEntry> ("Nereid.SearchEntry") ?? view_container.SearchEntry;
-            if (focus_search && search_entry.Visible && !search_entry.HasFocus && !source_view.EditingRow) {
-                search_entry.HasFocus = true;
+            if (focus_search && view_container.SearchEntry.Visible && !view_container.SearchEntry.HasFocus && !source_view.EditingRow) {
+                view_container.SearchEntry.HasFocus = true;
                 return true;
             }
 
