@@ -27,6 +27,7 @@
 //
 
 using System;
+using System.Collections.Generic;
 using System.Data;
 using System.IO;
 
@@ -38,8 +39,11 @@ using Hyena.Query;
 using Banshee.Base;
 using Banshee.Configuration.Schema;
 using Banshee.Database;
+using Banshee.Metadata;
+using Banshee.Preferences;
 using Banshee.Query;
 using Banshee.Sources;
+using Banshee.Library;
 using Banshee.ServiceStack;
 using Banshee.Streaming;
 
@@ -52,23 +56,16 @@ using Banshee.Streaming;
 
 namespace Banshee.Collection.Database
 {
-    public enum TrackUriType : int {
-        AbsolutePath = 0,
-        RelativePath = 1,
-        AbsoluteUri = 2
-    }
-
     public class DatabaseTrackInfo : TrackInfo
     {
         private static DatabaseTrackModelProvider<DatabaseTrackInfo> provider;
 
         public static DatabaseTrackModelProvider<DatabaseTrackInfo> Provider {
-            get { return provider ?? provider = new DatabaseTrackModelProvider<DatabaseTrackInfo> (ServiceManager.DbConnection); }
+            get { return provider ?? (provider = new DatabaseTrackModelProvider<DatabaseTrackInfo> (ServiceManager.DbConnection)); }
         }
 
         private bool artist_changed = false, album_changed = false;
-        private bool uri_fields_dirty = false;
-        
+
         public DatabaseTrackInfo () : base ()
         {
         }
@@ -78,19 +75,39 @@ namespace Banshee.Collection.Database
             Provider.Copy (original, this);
         }
 
-        public override void IncrementPlayCount ()
+        // Changing these fields shouldn't change DateUpdated (which triggers file save)
+        private static readonly HashSet<QueryField> transient_fields;
+
+        static DatabaseTrackInfo ()
         {
-            if (ProviderRefresh ()) {
-                base.IncrementPlayCount ();
-                Save (true, BansheeQuery.PlayCountField, BansheeQuery.LastPlayedField);
-            }
+            transient_fields = new HashSet<QueryField> () {
+                BansheeQuery.ScoreField,
+                BansheeQuery.SkipCountField,
+                BansheeQuery.LastSkippedField,
+                BansheeQuery.LastPlayedField,
+                BansheeQuery.PlaybackErrorField,
+                BansheeQuery.PlayCountField,
+                BansheeQuery.RatingField
+            };
+            Action<Root> handler = delegate {
+                if (SaveTrackMetadataService.WriteRatingsAndPlayCountsEnabled.Value) {
+                    transient_fields.Remove (BansheeQuery.PlayCountField);
+                    transient_fields.Remove (BansheeQuery.RatingField);
+                } else {
+                    transient_fields.Add (BansheeQuery.PlayCountField);
+                    transient_fields.Add (BansheeQuery.RatingField);
+                }
+            };
+            SaveTrackMetadataService.WriteRatingsAndPlayCountsEnabled.ValueChanged += handler;
+            handler (null);
         }
 
-        public override void IncrementSkipCount ()
+        public override void OnPlaybackFinished (double percentCompleted)
         {
-            if (ProviderRefresh ()) {
-                base.IncrementSkipCount ();
-                Save (true, BansheeQuery.SkipCountField, BansheeQuery.LastSkippedField);
+            if (ProviderRefresh()) {
+                base.OnPlaybackFinished (percentCompleted);
+                Save (true, BansheeQuery.ScoreField, BansheeQuery.SkipCountField, BansheeQuery.LastSkippedField,
+                    BansheeQuery.PlayCountField, BansheeQuery.LastPlayedField);
             }
         }
 
@@ -99,7 +116,7 @@ namespace Banshee.Collection.Database
             if (PrimarySource != null && PrimarySource.TrackEqualHandler != null) {
                 return PrimarySource.TrackEqualHandler (this, track);
             }
-            
+
             DatabaseTrackInfo db_track = track as DatabaseTrackInfo;
             if (db_track == null) {
                 return base.TrackEqual (track);
@@ -118,18 +135,15 @@ namespace Banshee.Collection.Database
 
         public static bool TrackEqual (DatabaseTrackInfo a, DatabaseTrackInfo b)
         {
-            return a != null && b != null && 
-                a.TrackId == b.TrackId && 
-                a.CacheModelId == b.CacheModelId && 
-                (int)a.CacheEntryId == (int)b.CacheEntryId;
+            return a != null && b != null && a.TrackId == b.TrackId;
         }
-        
+
         public DatabaseArtistInfo Artist {
-            get { return DatabaseArtistInfo.FindOrCreate (ArtistName); }
+            get { return DatabaseArtistInfo.FindOrCreate (ArtistName, ArtistNameSort); }
         }
 
         public DatabaseAlbumInfo Album {
-            get { return DatabaseAlbumInfo.FindOrCreate (DatabaseArtistInfo.FindOrCreate (AlbumArtist), AlbumTitle, IsCompilation); }
+            get { return DatabaseAlbumInfo.FindOrCreate (DatabaseArtistInfo.FindOrCreate (AlbumArtist, AlbumArtistSort), AlbumTitle, AlbumTitleSort, IsCompilation); }
         }
 
         private static bool notify_saved = true;
@@ -143,27 +157,38 @@ namespace Banshee.Collection.Database
             Save (NotifySaved);
         }
 
+        public override void UpdateLastPlayed ()
+        {
+            Refresh ();
+            base.UpdateLastPlayed ();
+            Save (NotifySaved, BansheeQuery.LastPlayedField);
+        }
+
         public void Save (bool notify, params QueryField [] fields_changed)
         {
-            // If either the artist or album changed, 
+            // If either the artist or album changed,
             if (ArtistId == 0 || AlbumId == 0 || artist_changed == true || album_changed == true) {
                 DatabaseArtistInfo artist = Artist;
                 ArtistId = artist.DbId;
-           
+
                 DatabaseAlbumInfo album = Album;
                 AlbumId = album.DbId;
-                
+
                 // TODO get rid of unused artists/albums
             }
-            
-            DateUpdated = DateTime.Now;
+
+            if (fields_changed.Length == 0 || !transient_fields.IsSupersetOf (fields_changed)) {
+                DateUpdated = DateTime.Now;
+            }
 
             bool is_new = (TrackId == 0);
-            if (is_new) DateAdded = DateUpdated;
+            if (is_new) {
+                DateAdded = DateUpdated = DateTime.Now;
+            }
 
             ProviderSave ();
 
-            if (notify) {
+            if (notify && PrimarySource != null) {
                 if (is_new) {
                     PrimarySource.NotifyTracksAdded ();
                 } else {
@@ -171,7 +196,7 @@ namespace Banshee.Collection.Database
                 }
             }
         }
-        
+
         protected virtual void ProviderSave ()
         {
             Provider.Save (this);
@@ -181,12 +206,12 @@ namespace Banshee.Collection.Database
         {
             ProviderRefresh ();
         }
-        
+
         protected virtual bool ProviderRefresh ()
         {
             return Provider.Refresh (this);
         }
-        
+
         private int track_id;
         [DatabaseColumn ("TrackID", Constraints = DatabaseColumnConstraints.PrimaryKey)]
         public int TrackId {
@@ -198,10 +223,7 @@ namespace Banshee.Collection.Database
         [DatabaseColumn ("PrimarySourceID")]
         public int PrimarySourceId {
             get { return primary_source_id; }
-            set {
-                primary_source_id = value;
-                UpdateUri ();
-            }
+            set { primary_source_id = value; }
         }
 
         public PrimarySource PrimarySource {
@@ -215,7 +237,7 @@ namespace Banshee.Collection.Database
             get { return artist_id; }
             set { artist_id = value; }
         }
-        
+
         private int album_id;
         [DatabaseColumn ("AlbumID")]
         public int AlbumId {
@@ -241,6 +263,24 @@ namespace Banshee.Collection.Database
             }
         }
 
+        [VirtualDatabaseColumn ("NameSort", "CoreArtists", "ArtistID", "ArtistID")]
+        protected string ArtistNameSortField {
+            get { return ArtistNameSort; }
+            set { base.ArtistNameSort = value; }
+        }
+
+        public override string ArtistNameSort {
+            get { return base.ArtistNameSort; }
+            set {
+                value = CleanseString (value, ArtistNameSort);
+                if (value == null)
+                    return;
+
+                base.ArtistNameSort = value;
+                artist_changed = true;
+            }
+        }
+
         [VirtualDatabaseColumn ("Title", "CoreAlbums", "AlbumID", "AlbumID")]
         protected string AlbumTitleField {
             get { return AlbumTitle; }
@@ -259,6 +299,24 @@ namespace Banshee.Collection.Database
             }
         }
 
+        [VirtualDatabaseColumn ("TitleSort", "CoreAlbums", "AlbumID", "AlbumID")]
+        protected string AlbumTitleSortField {
+            get { return AlbumTitleSort; }
+            set { base.AlbumTitleSort = value; }
+        }
+
+        public override string AlbumTitleSort {
+            get { return base.AlbumTitleSort; }
+            set {
+                value = CleanseString (value, AlbumTitleSort);
+                if (value == null)
+                    return;
+
+                base.AlbumTitleSort = value;
+                album_changed = true;
+            }
+        }
+
         [VirtualDatabaseColumn ("ArtistName", "CoreAlbums", "AlbumID", "AlbumID")]
         protected string AlbumArtistField {
             get { return AlbumArtist; }
@@ -276,7 +334,25 @@ namespace Banshee.Collection.Database
                 album_changed = true;
             }
         }
-        
+
+        [VirtualDatabaseColumn ("ArtistNameSort", "CoreAlbums", "AlbumID", "AlbumID")]
+        protected string AlbumArtistSortField {
+            get { return AlbumArtistSort; }
+            set { base.AlbumArtistSort = value; }
+        }
+
+        public override string AlbumArtistSort {
+            get { return base.AlbumArtistSort; }
+            set {
+                value = CleanseString (value, AlbumArtistSort);
+                if (value == null)
+                    return;
+
+                base.AlbumArtistSort = value;
+                album_changed = true;
+            }
+        }
+
         [VirtualDatabaseColumn ("IsCompilation", "CoreAlbums", "AlbumID", "AlbumID")]
         protected bool IsCompilationField {
             get { return IsCompilation; }
@@ -290,82 +366,46 @@ namespace Banshee.Collection.Database
                 album_changed = true;
             }
         }
-        
+
         private static string CleanseString (string input, string old_val)
         {
             if (input == old_val)
                 return null;
-                    
+
             if (input != null)
                 input = input.Trim ();
-            
+
             if (input == old_val)
                 return null;
-            
+
             return input;
         }
-        
+
         private int tag_set_id;
         [DatabaseColumn]
         public int TagSetID {
             get { return tag_set_id; }
             set { tag_set_id = value; }
         }
-        
+
         [DatabaseColumn ("MusicBrainzID")]
         public override string MusicBrainzId {
             get { return base.MusicBrainzId; }
             set { base.MusicBrainzId = value; }
         }
 
-        public override SafeUri Uri {
-            get { return base.Uri; }
-            set {
-                base.Uri = value;
-                uri_fields_dirty = true;
-            }
-        }
-        
-        private string uri_field;
         [DatabaseColumn ("Uri")]
         protected string UriField {
-            get {
-                if (uri_fields_dirty && Uri != null && PrimarySource != null) {
-                    PrimarySource.UriToFields (Uri, out uri_type, out uri_field);
-                    uri_fields_dirty = false;
-                }
-                return uri_field;
-            }
-            set {
-                uri_field = value;
-                UpdateUri ();
-            }
+            get { return Uri == null ? null : Uri.AbsoluteUri; }
+            set { Uri = value == null ? null : new SafeUri (value); }
         }
-        
-        private bool uri_type_set;
-        private TrackUriType uri_type;
-        [DatabaseColumn ("UriType")]
-        protected TrackUriType UriType {
-            get {
-                if (uri_fields_dirty && Uri != null && PrimarySource != null) {
-                    PrimarySource.UriToFields (Uri, out uri_type, out uri_field);
-                    uri_fields_dirty = false;
-                }
-                return uri_type;
-            }
-            set {
-                uri_type = value;
-                uri_type_set = true;
-                UpdateUri ();
-            }
-        }
-        
+
         [DatabaseColumn]
         public override string MimeType {
             get { return base.MimeType; }
             set { base.MimeType = value; }
         }
-        
+
         [DatabaseColumn]
         public override long FileSize {
             get { return base.FileSize; }
@@ -383,41 +423,52 @@ namespace Banshee.Collection.Database
             get { return base.LastSyncedStamp; }
             set { base.LastSyncedStamp = value; }
         }
-        
+
         [DatabaseColumn ("Attributes")]
         public override TrackMediaAttributes MediaAttributes {
             get { return base.MediaAttributes; }
             set { base.MediaAttributes = value; }
         }
-        
+
         [DatabaseColumn ("Title")]
         public override string TrackTitle {
             get { return base.TrackTitle; }
             set { base.TrackTitle = value; }
         }
-        
+
+        [DatabaseColumn ("TitleSort")]
+        public override string TrackTitleSort {
+            get { return base.TrackTitleSort; }
+            set { base.TrackTitleSort = value; }
+        }
+
+        [DatabaseColumn("TitleSortKey", Select = false)]
+        internal byte[] TrackTitleSortKey {
+            get { return Hyena.StringUtil.SortKey (TrackTitleSort ?? DisplayTrackTitle); }
+        }
+
         [DatabaseColumn(Select = false)]
-        protected string TitleLowered {
-            get { return TrackTitle == null ? null : TrackTitle.ToLower (); }
+        internal string TitleLowered {
+            get { return Hyena.StringUtil.SearchKey (DisplayTrackTitle); }
         }
 
         [DatabaseColumn(Select = false)]
         public override string MetadataHash {
             get { return base.MetadataHash; }
         }
-        
+
         [DatabaseColumn]
         public override int TrackNumber {
             get { return base.TrackNumber; }
             set { base.TrackNumber = value; }
         }
-        
+
         [DatabaseColumn]
         public override int TrackCount {
             get { return base.TrackCount; }
             set { base.TrackCount = value; }
         }
-        
+
         [DatabaseColumn ("Disc")]
         public override int DiscNumber {
             get { return base.DiscNumber; }
@@ -429,13 +480,13 @@ namespace Banshee.Collection.Database
             get { return base.DiscCount; }
             set { base.DiscCount = value; }
         }
-        
+
         [DatabaseColumn]
         public override TimeSpan Duration {
             get { return base.Duration; }
             set { base.Duration = value; }
         }
-        
+
         [DatabaseColumn]
         public override int Year {
             get { return base.Year; }
@@ -483,7 +534,7 @@ namespace Banshee.Collection.Database
             get { return base.Comment; }
             set { base.Comment = value; }
         }
-        
+
         [DatabaseColumn("BPM")]
         public override int Bpm {
             get { return base.Bpm; }
@@ -495,7 +546,19 @@ namespace Banshee.Collection.Database
             get { return base.BitRate; }
             set { base.BitRate = value; }
         }
-        
+
+        [DatabaseColumn]
+        public override int SampleRate {
+            get { return base.SampleRate; }
+            set { base.SampleRate = value; }
+        }
+
+        [DatabaseColumn]
+        public override int BitsPerSample {
+            get { return base.BitsPerSample; }
+            set { base.BitsPerSample = value; }
+        }
+
         [DatabaseColumn("Rating")]
         protected int rating;
         public override int Rating {
@@ -503,6 +566,12 @@ namespace Banshee.Collection.Database
             set { rating = value; }
         }
 
+        [DatabaseColumn]
+        public override int Score {
+            get { return base.Score; }
+            set { base.Score = value; }
+        }
+
         public int SavedRating {
             get { return rating; }
             set {
@@ -512,19 +581,19 @@ namespace Banshee.Collection.Database
                 }
             }
         }
-        
+
         [DatabaseColumn]
         public override int PlayCount {
             get { return base.PlayCount; }
             set { base.PlayCount = value; }
         }
-        
+
         [DatabaseColumn]
         public override int SkipCount {
             get { return base.SkipCount; }
             set { base.SkipCount = value; }
         }
-        
+
         private long external_id;
         [DatabaseColumn ("ExternalID")]
         public long ExternalId {
@@ -541,7 +610,7 @@ namespace Banshee.Collection.Database
                 return external_object;
             }
         }
-        
+
         [DatabaseColumn ("LastPlayedStamp")]
         public override DateTime LastPlayed {
             get { return base.LastPlayed; }
@@ -553,7 +622,7 @@ namespace Banshee.Collection.Database
             get { return base.LastSkipped; }
             set { base.LastSkipped = value; }
         }
-        
+
         [DatabaseColumn ("DateAddedStamp")]
         public override DateTime DateAdded {
             get { return base.DateAdded; }
@@ -566,7 +635,7 @@ namespace Banshee.Collection.Database
             get { return date_updated; }
             set { date_updated = value; }
         }
-        
+
         [DatabaseColumn ("LastStreamError")]
         protected StreamPlaybackError playback_error;
         public override StreamPlaybackError PlaybackError {
@@ -575,73 +644,87 @@ namespace Banshee.Collection.Database
                 if (playback_error == value) {
                     return;
                 }
-                
-                playback_error = value; 
+
+                playback_error = value;
             }
         }
 
-        private void UpdateUri ()
-        {
-            if (Uri == null && uri_type_set && UriField != null && PrimarySource != null) {
-                Uri = PrimarySource.UriAndTypeToSafeUri (UriType, UriField);
+        public PathPattern PathPattern {
+            get {
+                var src = PrimarySource;
+                var pattern = src == null ? null : src.PathPattern;
+                return pattern ?? MusicLibrarySource.MusicFileNamePattern;
             }
         }
 
-        public void CopyToLibraryIfAppropriate (bool force_copy)
+        public bool CopyToLibraryIfAppropriate (bool force_copy)
         {
+            bool copy_success = true;
+
             SafeUri old_uri = this.Uri;
             if (old_uri == null) {
                 // Get out quick, no URI set yet.
-                return;
+                return copy_success;
             }
-            
-            bool in_library = old_uri.AbsolutePath.StartsWith (Paths.CachedLibraryLocationWithSeparator);
+
+            bool in_library = old_uri.IsLocalPath ? old_uri.AbsolutePath.StartsWith (PrimarySource.BaseDirectoryWithSeparator) : false;
 
             if (!in_library && (LibrarySchema.CopyOnImport.Get () || force_copy)) {
-                string new_filename = FileNamePattern.BuildFull (this, Path.GetExtension (old_uri.ToString ()));
+                string new_filename = PathPattern.BuildFull (PrimarySource.BaseDirectory, this, Path.GetExtension (old_uri.ToString ()));
                 SafeUri new_uri = new SafeUri (new_filename);
 
                 try {
                     if (Banshee.IO.File.Exists (new_uri)) {
-                        return;
+                        if (Banshee.IO.File.GetSize (old_uri) == Banshee.IO.File.GetSize (new_uri)) {
+                            Hyena.Log.DebugFormat ("Not copying {0} to library because there is already a file of same size at {1}", old_uri, new_uri);
+                            copy_success = false;
+                            return copy_success;
+                        } else {
+                            string extension = Path.GetExtension (new_filename);
+                            string filename_no_ext = new_filename.Remove (new_filename.Length - extension.Length);
+                            int duplicate_index = 1;
+                            while (Banshee.IO.File.Exists (new_uri)) {
+                                new_filename = String.Format ("{0} ({1}){2}", filename_no_ext, duplicate_index, extension);
+                                new_uri = new SafeUri (new_filename);
+                                duplicate_index++;
+                          }
+                        }
                     }
-                    
+
                     Banshee.IO.File.Copy (old_uri, new_uri, false);
                     Uri = new_uri;
                 } catch (Exception e) {
                     Log.ErrorFormat ("Exception copying into library: {0}", e);
                 }
             }
+            return copy_success;
         }
 
-        private static HyenaSqliteCommand check_command = new HyenaSqliteCommand (
-            "SELECT TrackID FROM CoreTracks WHERE PrimarySourceId IN (?) AND (Uri = ? OR Uri = ?) LIMIT 1"
+        private static HyenaSqliteCommand get_uri_id_cmd = new HyenaSqliteCommand ("SELECT TrackID FROM CoreTracks WHERE Uri = ? LIMIT 1");
+        public static int GetTrackIdForUri (string uri)
+        {
+            return ServiceManager.DbConnection.Query<int> (get_uri_id_cmd, new SafeUri (uri).AbsoluteUri);
+        }
+
+        private static HyenaSqliteCommand get_track_id_by_uri = new HyenaSqliteCommand (
+            "SELECT TrackID FROM CoreTracks WHERE PrimarySourceId IN (?) AND Uri = ? LIMIT 1"
         );
 
-        public static int GetTrackIdForUri (SafeUri uri, string relative_path, int [] primary_sources)
+        public static int GetTrackIdForUri (SafeUri uri, int [] primary_sources)
         {
-            return ServiceManager.DbConnection.Query<int> (check_command,
-                primary_sources, relative_path, uri.AbsoluteUri);
+            return ServiceManager.DbConnection.Query<int> (get_track_id_by_uri,
+                primary_sources, uri.AbsoluteUri);
         }
 
-        public static int GetTrackIdForUri (string relative_path, int [] primary_sources)
+        public static int GetTrackIdForUri (string absoluteUri, int [] primary_sources)
         {
-            return GetTrackIdForUri (relative_path, relative_path, primary_sources);
-        }
-        
-        public static int GetTrackIdForUri (string uri, string relative_path, int [] primary_sources)
-        {
-            return ServiceManager.DbConnection.Query<int> (check_command, primary_sources, uri, relative_path);
-        }
-        
-        public static bool ContainsUri (string relative_path, int [] primary_sources)
-        {
-            return GetTrackIdForUri (relative_path, primary_sources) > 0;
+            return ServiceManager.DbConnection.Query<int> (get_track_id_by_uri,
+                primary_sources, absoluteUri);
         }
 
-        public static bool ContainsUri (SafeUri uri, string relative_path, int [] primary_sources)
+        public static bool ContainsUri (SafeUri uri, int [] primary_sources)
         {
-            return GetTrackIdForUri (uri, relative_path, primary_sources) > 0;
+            return GetTrackIdForUri (uri, primary_sources) > 0;
         }
     }
 }
