@@ -30,20 +30,23 @@ using System;
 using System.Data;
 using Banshee.Collection;
 using Hyena.Data.Sqlite;
+using Hyena.Query;
 
 namespace Banshee.Collection.Database
 {
     public interface IDatabaseTrackModelCache
     {
         void Clear ();
-        void ClearManagedCache ();
         void SaveSelection ();
         void UpdateAggregates ();
         void RestoreSelection ();
         long Count { get; }
         void Reload ();
-        long IndexOf (long item_entry_id);
-        TrackInfo GetSingle (string random_fragment, params object [] args);
+        long IndexOf (string where_fragment, long offset);
+        long IndexOf (Hyena.Data.ICacheableItem item);
+        long IndexOf (object item_entry_id);
+        TrackInfo GetSingle (string selectFragment, string fromFragment, string conditionOrderFragment, params object [] args);
+        TrackInfo GetSingleWhere (string conditionOrderFragment, params object [] args);
         TrackInfo GetValue (long index);
         long CacheId { get; }
         event Action<IDataReader> AggregatesUpdated;
