@@ -32,6 +32,7 @@ using System.Net;
 using System.Xml;
 using System.Text;
 using System.Collections.Generic;
+using System.Web;
 
 using Hyena;
 using Banshee.Base;
@@ -39,97 +40,104 @@ using Banshee.Metadata;
 using Banshee.Kernel;
 using Banshee.Collection;
 using Banshee.Streaming;
+using Banshee.Networking;
 
 namespace Banshee.Metadata.MusicBrainz
 {
     public class MusicBrainzQueryJob : MetadataServiceJob
     {
         private static string AmazonUriFormat = "http://images.amazon.com/images/P/{0}.01._SCLZZZZZZZ_.jpg";
-    
-        private TrackInfo track;
+
         private string asin;
-        
-        public MusicBrainzQueryJob(IBasicTrackInfo track, MetadataSettings settings)
+
+        public MusicBrainzQueryJob (IBasicTrackInfo track)
         {
             Track = track;
-            this.track = track as TrackInfo; 
-            Settings = settings;
         }
-        
-        public MusicBrainzQueryJob(IBasicTrackInfo track, MetadataSettings settings, string asin) : this(track, settings)
+
+        public MusicBrainzQueryJob (IBasicTrackInfo track, string asin) : this (track)
         {
             this.asin = asin;
         }
-        
-        public override void Run()
+
+        public override void Run ()
         {
-            Lookup();
+            Lookup ();
         }
-        
-        public bool Lookup()
+
+        public bool Lookup ()
         {
-            if(track == null) {
+            if (Track == null || (Track.MediaAttributes & TrackMediaAttributes.Podcast) != 0) {
                 return false;
             }
-            
-            string album_artist_id = track.ArtistAlbumId;
-            
-            if(album_artist_id == null) {
+
+            string artwork_id = Track.ArtworkId;
+
+            if (artwork_id == null) {
                 return false;
-            } else if(CoverArtSpec.CoverExists(album_artist_id)) {
+            } else if (CoverArtSpec.CoverExists (artwork_id)) {
                 return false;
-            } else if(!Settings.NetworkConnected) {
+            } else if (!InternetConnected) {
                 return false;
             }
-            
-            if(asin == null) {
-                asin = FindAsin();
-                if(asin == null) {
+
+            if (asin == null) {
+                asin = FindAsin ();
+                if (asin == null) {
                     return false;
                 }
             }
-            
-            if(SaveHttpStreamCover(new Uri(String.Format(AmazonUriFormat, asin)), album_artist_id, 
-                new string [] { "image/gif" })) {
-                Log.Debug ("Downloaded cover art from Amazon", album_artist_id);
-                StreamTag tag = new StreamTag();
-                tag.Name = CommonTags.AlbumCoverId;
-                tag.Value = album_artist_id;
 
-                AddTag(tag);
-                
+            if (SaveHttpStreamCover (new Uri (String.Format (AmazonUriFormat, asin)), artwork_id,
+                new string [] { "image/gif" })) {
+                Log.Debug ("Downloaded cover art from Amazon", artwork_id);
+                StreamTag tag = new StreamTag ();
+                tag.Name = CommonTags.AlbumCoverId;
+                tag.Value = artwork_id;
+
+                AddTag (tag);
+
                 return true;
             }
-            
+
             return false;
         }
 
-        // MusicBrainz has this new XML API, so I'm using that here 
-        // instead of using the stuff in the MusicBrainz namespace 
+        // MusicBrainz has this new XML API, so I'm using that here
+        // instead of using the stuff in the MusicBrainz namespace
         // which sucks and doesn't even appear to work anymore.
-        
-        private string FindAsin()
+
+        private string FindAsin ()
         {
-            Uri uri = new Uri(String.Format("http://musicbrainz.org/ws/1/release/?type=xml&artist={0}&title={1}",
-                track.ArtistName, track.AlbumTitle));
+            string album_artist = HttpUtility.UrlEncode (Track.AlbumArtist);
+            string album_title = HttpUtility.UrlEncode (Track.AlbumTitle);
+            Uri uri = new Uri (String.Format ("http://musicbrainz.org/ws/1/release/?type=xml&artist={0}&title={1}",
+                album_artist, album_title));
 
-            XmlTextReader reader = new XmlTextReader(GetHttpStream(uri));
+            HttpWebResponse response = GetHttpStream (uri);
+            if (response == null) {
+                return null;
+            }
 
-            bool haveMatch = false;
-            
-            while(reader.Read()) {
-                if(reader.NodeType == XmlNodeType.Element) {
-                    switch (reader.LocalName) {
-                        case "release":
-                            haveMatch = reader["ext:score"] == "100";
+            using (Stream stream = response.GetResponseStream ()) {
+                XmlTextReader reader = new XmlTextReader (stream);
+
+                bool haveMatch = false;
+
+                while (reader.Read ()) {
+                    if (reader.NodeType == XmlNodeType.Element) {
+                        switch (reader.LocalName) {
+                            case "release":
+                                haveMatch = reader["ext:score"] == "100";
+                                break;
+                            case "asin":
+                                if (haveMatch) {
+                                    return reader.ReadString ();
+                                }
                             break;
-                        case "asin":
-                            if(haveMatch) {
-                                return reader.ReadString();
-                            }
-                        break;
-                    default:
-                        break;
+                        default:
+                            break;
+                        }
                     }
                 }
             }
