@@ -38,13 +38,13 @@ namespace Banshee.Gui.TrackEditor
 
         public event EventHandler Changed;
 
-        private SpinButtonEntry from_entry;
-        public SpinButtonEntry From {
+        private SpinButton from_entry;
+        public SpinButton From {
             get { return from_entry; }
         }
 
-        private SpinButtonEntry to_entry;
-        public SpinButtonEntry To {
+        private SpinButton to_entry;
+        public SpinButton To {
             get { return to_entry; }
         }
 
@@ -76,12 +76,6 @@ namespace Banshee.Gui.TrackEditor
             to_entry.ValueChanged += OnChanged;
         }
 
-        public void SetAsReadOnly ()
-        {
-            from_entry.IsEditable = false;
-            to_entry.IsEditable = false;
-        }
-
         protected override bool OnMnemonicActivated (bool group_cycling) {
             return from_entry.MnemonicActivate(group_cycling);
         }
