@@ -91,35 +91,31 @@ namespace Banshee.Gui
                 new ActionEntry ("HelpMenuAction", null,
                     Catalog.GetString ("_Help"), null, null, null),
 
-                new ActionEntry ("UserHelp", Gtk.Stock.Help,
-                    Catalog.GetString ("_Contents"), "F1", null,
-                    delegate { Banshee.ServiceStack.Application.DisplayHelp (null); }),
-
                 new ActionEntry ("WebMenuAction", null,
                     Catalog.GetString ("_Web Resources"), null, null, null),
 
                 new ActionEntry ("WikiGuideAction", Stock.Help,
                     Catalog.GetString ("Banshee _User Guide (Wiki)"), null,
                     Catalog.GetString ("Learn about how to use Banshee"), delegate {
-                        Banshee.Web.Browser.Open ("http://banshee.fm/support/guide/");
+                        Banshee.Web.Browser.Open ("http://banshee-project.org/support/guide/");
                     }),
 
                 new ActionEntry ("WikiSearchHelpAction", null,
                     Catalog.GetString ("Advanced Collection Searching"), null,
                     Catalog.GetString ("Learn advanced ways to search your media collection"), delegate {
-                        Banshee.Web.Browser.Open ("http://banshee.fm/support/guide/searching/");
+                        Banshee.Web.Browser.Open ("http://banshee-project.org/support/guide/searching/");
                     }),
 
                 new ActionEntry ("WikiAction", null,
                     Catalog.GetString ("Banshee _Home Page"), null,
                     Catalog.GetString ("Visit the Banshee Home Page"), delegate {
-                        Banshee.Web.Browser.Open ("http://banshee.fm/");
+                        Banshee.Web.Browser.Open ("http://banshee-project.org/");
                     }),
 
                 new ActionEntry ("WikiDeveloperAction", null,
                     Catalog.GetString ("_Get Involved"), null,
                     Catalog.GetString ("Become a contributor to Banshee"), delegate {
-                        Banshee.Web.Browser.Open ("http://banshee.fm/contribute/");
+                        Banshee.Web.Browser.Open ("http://banshee-project.org/contribute/");
                     }),
 
                 new ActionEntry ("VersionInformationAction", null,
