@@ -34,6 +34,7 @@ using Cairo;
 using Mono.Unix;
 
 using Hyena;
+using Hyena.Gui;
 using Hyena.Gui.Theming;
 using Hyena.Gui.Theatrics;
 
@@ -59,7 +60,7 @@ namespace Banshee.Sources.Gui
         private Stage<TreeIter> notify_stage = new Stage<TreeIter> (2000);
 
         private TreeViewColumn focus_column;
-        private TreePath highlight_path;
+        private TreeIter highlight_iter = TreeIter.Zero;
         private SourceModel store;
         private int current_timeout = -1;
         private bool editing_row = false;
@@ -80,6 +81,8 @@ namespace Banshee.Sources.Gui
             ConnectEvents ();
 
             RowSeparatorFunc = RowSeparatorHandler;
+
+            ResetSelection ();
         }
 
 #region Setup Methods
@@ -131,8 +134,10 @@ namespace Banshee.Sources.Gui
                     return false;
                 }
 
-                Gdk.Rectangle rect = GetBackgroundArea (store.GetPath (actor.Target), focus_column);
-                QueueDrawArea (rect.X, rect.Y, rect.Width, rect.Height);
+                using (var path = store.GetPath (actor.Target) ) {
+                    Gdk.Rectangle rect = GetBackgroundArea (path, focus_column);
+                    QueueDrawArea (rect.X, rect.Y, rect.Width, rect.Height);
+                }
                 return true;
             };
         }
@@ -141,12 +146,10 @@ namespace Banshee.Sources.Gui
 
 #region Gtk.Widget Overrides
 
-        protected override void OnRealized ()
+        protected override void OnStyleSet (Style old_style)
         {
-            base.OnRealized ();
-
-            theme = new GtkTheme (this);
-            // theme.RefreshColors ();
+            base.OnStyleSet (old_style);
+            theme = Hyena.Gui.Theming.ThemeEngine.CreateTheme (this);
         }
 
         protected override bool OnButtonPressEvent (Gdk.EventButton press)
@@ -192,9 +195,12 @@ namespace Banshee.Sources.Gui
             }
 
             if (press.Button == 3) {
-                HighlightPath (path);
-                OnPopupMenu ();
-                return true;
+                TreeIter iter;
+                if (Model.GetIter (out iter, path)) {
+                    HighlightIter (iter);
+                    OnPopupMenu ();
+                    return true;
+                }
             }
 
             if (!source.CanActivate) {
@@ -243,10 +249,14 @@ namespace Banshee.Sources.Gui
 
             try {
                 cr = Gdk.CairoHelper.Create (evnt.Window);
-                return base.OnExposeEvent (evnt);
+                base.OnExposeEvent (evnt);
+                if (Hyena.PlatformDetection.IsMeeGo) {
+                    theme.DrawFrameBorder (cr, new Gdk.Rectangle (0, 0,
+                        Allocation.Width, Allocation.Height));
+                }
+                return true;
             } finally {
-                ((IDisposable)cr.Target).Dispose ();
-                ((IDisposable)cr).Dispose ();
+                CairoExtensions.DisposeContext (cr);
                 cr = null;
             }
         }
@@ -334,8 +344,9 @@ namespace Banshee.Sources.Gui
 
         private void Expand (TreeIter iter)
         {
-            TreePath path = store.GetPath (iter);
-            ExpandRow (path, true);
+            using (var path = store.GetPath (iter)) {
+                ExpandRow (path, true);
+            }
         }
 
         private void OnSourceUserNotifyUpdated (object o, EventArgs args)
@@ -392,7 +403,9 @@ namespace Banshee.Sources.Gui
             }
 
             renderer.Editable = true;
-            SetCursor (store.GetPath (iter), focus_column, true);
+            using (var path = store.GetPath (iter)) {
+                SetCursor (path, focus_column, true);
+            }
             renderer.Editable = false;
         }
 
@@ -405,15 +418,15 @@ namespace Banshee.Sources.Gui
             }
         }
 
-        public void HighlightPath (TreePath path)
+        public void HighlightIter (TreeIter iter)
         {
-            highlight_path = path;
+            highlight_iter = iter;
             QueueDraw ();
         }
 
         public void ResetHighlight ()
         {
-            highlight_path = null;
+            highlight_iter = TreeIter.Zero;
             QueueDraw ();
         }
 
@@ -423,13 +436,11 @@ namespace Banshee.Sources.Gui
 
         public Source HighlightedSource {
             get {
-                TreeIter iter;
-
-                if (highlight_path == null || !store.GetIter (out iter, highlight_path)) {
+                if (TreeIter.Zero.Equals (highlight_iter)) {
                     return null;
                 }
 
-                return store.GetValue (iter, 0) as Source;
+                return store.GetValue (highlight_iter, 0) as Source;
             }
         }
 
@@ -445,8 +456,8 @@ namespace Banshee.Sources.Gui
 
 #region Internal Properties
 
-        internal TreePath HighlightedPath {
-            get { return highlight_path; }
+        internal TreeIter HighlightedIter {
+            get { return highlight_iter; }
         }
 
         internal Cairo.Context Cr {
