@@ -211,8 +211,6 @@ namespace Banshee.Dap
                     throw new PossibleUserErrorException (to_remove.Count);
                 }
 
-                Hyena.Log.Debug ("Starting DAP sync");
-
                 sync.Dap.DeleteAllTracks (to_remove);
                 sync.Dap.AddAllTracks (to_add);
 
@@ -246,8 +244,6 @@ namespace Banshee.Dap
                     }
                 }
 
-                Hyena.Log.Debug ("Ending DAP sync");
-
                 CalculateSync ();
                 sync.OnUpdated ();
             }
