@@ -40,41 +40,51 @@ using Banshee.Sources.Gui;
 namespace Banshee.NowPlaying
 {
     public class NowPlayingInterface : VBox, ISourceContents
-    {   
+    {
         private NowPlayingSource source;
         private Hyena.Widgets.RoundedFrame frame;
         private Gtk.Window video_window;
         private FullscreenAdapter fullscreen_adapter;
         private ScreensaverManager screensaver;
-        private NowPlayingContents contents;
         
+        internal NowPlayingContents Contents { get; private set; }
+
         public NowPlayingInterface ()
         {
             GtkElementsService service = ServiceManager.Get<GtkElementsService> ();
-            
-            contents = new NowPlayingContents ();
-            
+
+            Contents = new NowPlayingContents ();
+            Contents.ButtonPressEvent += (o, a) => {
+                if (a.Event.Type == Gdk.EventType.TwoButtonPress) {
+                    var iaservice = ServiceManager.Get<InterfaceActionService> ();
+                    var action = iaservice.ViewActions["FullScreenAction"] as Gtk.ToggleAction;
+                    if (action != null && action.Sensitive) {
+                        action.Active = !action.Active;
+                    }
+                }
+            };
+
             // This is my really sweet hack - it's where the video widget
             // is sent when the source is not active. This keeps the video
             // widget from being completely destroyed, causing problems with
             // its internal windowing and GstXOverlay. It's also conveniently
-            // the window that is used to do fullscreen video. Sweeeeeeeeeet. 
+            // the window that is used to do fullscreen video. Sweeeeeeeeeet.
             video_window = new FullscreenWindow (service.PrimaryWindow);
             video_window.Hidden += OnFullscreenWindowHidden;
             video_window.Realize ();
-            video_window.Add (contents);
-            
+            video_window.Add (Contents);
+
             frame = new Hyena.Widgets.RoundedFrame ();
             frame.SetFillColor (new Cairo.Color (0, 0, 0));
             frame.DrawBorder = false;
             frame.Show ();
-            
+
             PackStart (frame, true, true, 0);
-            
+
             fullscreen_adapter = new FullscreenAdapter ();
             screensaver = new ScreensaverManager ();
         }
-        
+
         public override void Dispose ()
         {
             base.Dispose ();
@@ -83,26 +93,26 @@ namespace Banshee.NowPlaying
 
         private void MoveVideoExternal (bool hidden)
         {
-            if (contents.Parent != video_window) {
-                contents.Visible = !hidden;
-                contents.Reparent (video_window);
+            if (Contents.Parent != video_window) {
+                Contents.Visible = !hidden;
+                Contents.Reparent (video_window);
             }
         }
-        
+
         private void MoveVideoInternal ()
         {
-            if (contents.Parent != frame) {
-                contents.Reparent (frame);
-                contents.Show ();
+            if (Contents.Parent != frame) {
+                Contents.Reparent (frame);
+                Contents.Show ();
             }
         }
-        
+
         protected override void OnRealized ()
         {
             base.OnRealized ();
             MoveVideoInternal ();
         }
-        
+
         protected override void OnUnrealized ()
         {
             MoveVideoExternal (false);
@@ -125,12 +135,12 @@ namespace Banshee.NowPlaying
         internal void OverrideFullscreen ()
         {
             FullscreenHandler (false);
-            
-            InterfaceActionService service = ServiceManager.Get<InterfaceActionService> (); 
+
+            InterfaceActionService service = ServiceManager.Get<InterfaceActionService> ();
             if (service == null || service.ViewActions == null) {
                 return;
             }
-            
+
             previous_fullscreen_handler = service.ViewActions.Fullscreen;
             service.ViewActions.Fullscreen = FullscreenHandler;
             DisableFullscreenAction ();
@@ -139,15 +149,15 @@ namespace Banshee.NowPlaying
         internal void RelinquishFullscreen ()
         {
             FullscreenHandler (false);
-            
-            InterfaceActionService service = ServiceManager.Get<InterfaceActionService> (); 
+
+            InterfaceActionService service = ServiceManager.Get<InterfaceActionService> ();
             if (service == null || service.ViewActions == null) {
                 return;
             }
-            
+
             service.ViewActions.Fullscreen = previous_fullscreen_handler;
         }
-        
+
         private void OnFullscreenWindowHidden (object o, EventArgs args)
         {
             MoveVideoInternal ();
@@ -168,11 +178,11 @@ namespace Banshee.NowPlaying
                 video_window.Hide ();
             }
         }
-        
+
 #endregion
-        
+
 #region ISourceContents
-        
+
         public bool SetSource (ISource src)
         {
             this.source = source as NowPlayingSource;
@@ -191,7 +201,7 @@ namespace Banshee.NowPlaying
         public Widget Widget {
             get { return this; }
         }
-        
+
 #endregion
 
     }
